#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Manipulation de séries financières avec la classe StockPrices
# 
# La classe [StockPrices](http://www.xavierdupre.fr/app/pyensae/helpsphinx/notebooks/pyensae%20StockPrices.html?highlight=stockprices) facilite la récupération de données financières via différents sites. Le site [Yahoo Finance](http://finance.yahoo.com/) requiet maintenant un cookie (depuis Mai 2017) et il est préférable de choisir [Google](https://www.google.com/finance?q=apple&ei=bPghWZnLDoKVULfBk-gC) ou [Quandl](https://www.quandl.com/tools/python). Google ne fonctionne que les marchés américains, [Quandl](https://www.quandl.com/tools/python) a des historiques plus courts.

# In[1]:


import pyensae
from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


# ### Initialisation

# In[3]:


import pyensae
import os
from pyensae.finance import StockPrices
cache = os.path.abspath("cache")
if not os.path.exists(cache):
    os.mkdir(cache)


# ### Créer un objet StockPrices

# Le plus est d'utiliser le tick de la série financière utilisé par le site [Yahoo Finance](https://fr.finance.yahoo.com) ou [Google Finance](https://www.google.com/search?q=finance) qui fait maintenant partie du moteur de recherche ou [quandl](https://www.quandl.com/tools/python).

# In[4]:


source = 'yahoo_new'
tick = 'MSFT'
stock = StockPrices(tick, folder=cache, url=source)
stock.head()


# In[5]:


stock.tail()


# La classe <tt>StockPrices</tt> contient un objet <a href="http://pandas.pydata.org/pandas-docs/version/0.13.1/generated/pandas.DataFrame.html">pandas.DataFrame</a> auquel on accède en écrivant <tt>stock.dataframe</tt> ou <tt>stock.df</tt> :

# In[6]:


stock.dataframe.columns


# De la même manière, on peut créer un objet <tt>StockPrices</tt> à partir d'un DataFrame :

# In[7]:


import pandas
data = [{"Date":"2014-04-01", "Close":105.6}, {"Date":"2014-04-02", "Close":104.6}, 
        {"Date":"2014-04-03", "Close":105.8}, ]
df = pandas.DataFrame(data)
stock = StockPrices("donnees",df)
stock.head()


# ### Quelques graphes

# Premier dessin, on télécharge les données de BNP puis on dessine le cours de l'action.

# In[8]:


import datetime
stock = StockPrices(tick, folder=cache, url=source)
ax = StockPrices.draw(stock, figsize=(12,6))


# In[9]:


stock = StockPrices(tick, folder=cache, url=source)
StockPrices.draw(stock, figsize=(12,6));


# La même chose se produit sur une autre série financière mais pas à la même date. On trace maintenant la série *Open* (*Adj Close* défini
# sur cette page [View and download historical price, dividend, or split data](https://help.yahoo.com/kb/finance/historical-prices-sln2311.html?impressions=true) n'est disponible qu'avec Yahoo).

# In[10]:


stock = StockPrices("MSFT", folder=cache, url='yahoo')
StockPrices.draw(stock, field=["Open", "Close"], figsize=(12,6));


# Ce type de série ne fait pas toujours apparaître les saut de prix qui survient comme par exemple le <a href="http://invest.bnpparibas.com/fr/pid5900/en-bref.html">20 février 2002</a> lorsque le cours nominal de l'action de la BNP a été divisé par deux pour augmenter la liquidité. Le nombre d'actions a été multiplié par deux. Les données sont le plus souvent corrigées [BNP février 2002](http://www.google.com/finance/historical?cid=667923&startdate=Feb+15%2C+2002&enddate=Feb+28%2C+2002&num=30&ei=AP8hWaDXF4WNUNuujYgK).

# ### Ajouter une seconde série sur un graphe

# Dans l'exemple suivant, on trace une série financière puis on ajoute la série des rendements sur un second axe.

# In[11]:


stock.head()


# In[12]:


stock = StockPrices(tick)
ret = stock.returns()["2019-01-04":"2019-02-02"]
ret.dataframe.loc["2019-01-11":"2019-01-18","Close"] = 0  # on annule certains valeurs
ax = stock.plot(figsize=(16, 5))
ret.plot(axis=2, ax=ax, label_prefix="r", color='blue');


# ### Quelques opérations

# In[13]:


os.listdir(cache)


# On affiche les dernières lignes.

# In[14]:


stock.tail()


# On récupère la série des rendements.

# In[15]:


ret = stock.returns()
ret.tail()


# On trace la série des rendements pour les derniers mois.

# In[16]:


StockPrices.draw(ret, figsize=(12,6), begin="2013-12-01", date_format="%Y-%m");


# ### Quelques notions sur les dates

# La classe <i>StockPrices</i> utilise les dates sous forme de chaînes de caractères. De cette façon, il n'est pas possible de faire des opérations dessus. Pour ce faire, il faut les convertir en un objet appelé <a href="https://docs.python.org/2/library/datetime.html">datetime</a>.

# In[17]:


from datetime import datetime, timedelta
dt = datetime.strptime("2014-03-31","%Y-%m-%d")
dt


# On ajoute un jour :

# In[18]:


delta = timedelta(1)
dt = dt + delta
dt


# Puis on convertit dans l'autre sens :

# In[19]:


s = dt.strftime("%Y-%m-%d")
s


# ### Promenade dans l'index

# Il est facile de récupérer les valeurs correspondant à une date précise. Mais comment récupérer la valeur du jour d'après ?

# In[20]:


tick2 = 'GOOGL'
stock = StockPrices(tick2, folder=cache, url=source)
df = stock.dataframe
print("A", df["2005-01-04":"2005-01-06"])
print("D", df.loc["2005-01-04","Close"])
print("G", df.index.get_loc("2005-01-06"))  # retourne la position de cette date


# ### Sauver les tables

# On peut conserver les données sous forme de fichiers pour les récupérer plus tard.

# In[21]:


stock = StockPrices(tick2, folder=cache, url=source)
stock.dataframe.to_csv("donnees.txt", sep="\t")
[_ for _ in os.listdir(".") if "donnees" in _]


# Le fichier est sauvé. Pour le récupérer avec pandas :

# In[22]:


import pandas
df = pandas.read_csv("donnees.txt", sep="\t")
df.head()


# Les dates apparaissent deux fois.

# In[23]:


with open("donnees.txt","r") as f: 
    text = f.read()
print(text[:400])


# Cela est dû au fait que les dates sont à la fois une colonne et servent d'index. Pour éviter de les conserver deux fois, on demande explicitement à ce que l'index ne soit pas ajouté au fichier : 

# In[24]:


stock = StockPrices(tick2, folder=cache, url=source)
stock.dataframe.to_csv("donnees.txt", sep="\t", index=False)


# Puis on récupère les données :

# In[25]:


df = pandas.read_csv("donnees.txt",sep="\t")
df.head()


# On vérifie le fichier sur disque dur :

# In[26]:


with open("donnees.txt", "r") as f:
    text = f.read()
print(text[:400])


# C'est mieux.

# In[27]: